# 🚀 programmering.no | 🤓 matematikk.as
# - Se om intervallet har fibonacci-tall
# - Fibonacci-rekken genereres av formelen F(n) = F(n-2) + F(n-1)
# - De første fibonacci-tallene er 0, 1, 1, 2, 3, 5, 8, 13, 21, ..., n
# - F(n) er også et fibonacci-tall hvis 5 * n^2 ± 4 er et kvadrat-tall

import math

# Verdier
min = 1
max = 5

# Funksjon for å se om n er et kvadrat-tall
def er_kvadrattall(n = int()):

    # Ta kvadrat-roten av n
    rot = math.sqrt(float(n))

    # Rund av roten til et heltall
    rot_heltall = int(rot)

    # Lag et nytt kvadrat-tall av de to røttene
    kvadrat = rot_heltall * rot_heltall

    # Hvis det nye kvadrat-tallet er lik n => n er et kvadrat-tall
    if kvadrat == n: return True

    # Hvis det nye kvadrat-tallet ikke er lik n => n er ikke et kvadrat-tall
    if kvadrat != n: return False

# Funksjon for å se om tallet er et fibonacci-tall
def er_fibonacci_tall(n):

    # Ikke definert
    if n < 1: return False

    # Tallet n er et fibonacci-tall hvis 5 * n^2 ± 4 er et kvadrattall
    er_fib_true_1 = er_kvadrattall(5 * int(n)**2 + 4)
    er_fib_true_2 = er_kvadrattall(5 * int(n)**2 - 4)

    # Ett eller begge er fibonacci-tall
    if er_fib_true_1 or er_fib_true_2: return True

    # Ingen er fibonacci-tall
    else: return False

# Funksjon for å printe fibonacci-tallene
def er_fibonacci_tall_print(n):

    # Print
    if er_fibonacci_tall(n) == True: print(f"Er {n} et fibonacci-tall? Ja")
    else:                            print(f"Er {n} et fibonacci-tall? Nei")

# Funksjon for å se om intervallet har fibonacci-tall
def er_fibonacci_tall_intervall(min, max):

    # Itererer gjennom intervallet
    for n in range(int(min), int(max) + 1):
        er_fibonacci_tall_print(n)

# Fibonacci-tall eller ikke
er_fibonacci_tall_intervall(min, max)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 3 c) Coca-Cola vs. Pepsi-Cola - 20 glass 

from scipy.stats import binom
import numpy as np

_blokk = 1
if _blokk == 1:
    n                       = 30      # Antall forsøk
    p                       = 0.5     # Sannsynlighet (sannsyn) for riktig cola-type
    x_liste                 = []      # Liste med alle mulige x-verdier
    x_riktig                = 0       # Antall riktige
    fordeling_liste         = []      # Liste med sannsynlighetsfordelingen til x
    fordeling_liste_kum     = []      # Liste med kumulativ sannsynlighetsfordeling
    sannsyn_riktig_kum      = 1       # Starter på 1 og trekker fra økende x_riktig til liten nok konfidens
    konfidens               = 0.05    # Konfidens
    desimal                 = 3       # Antall desimaler i avrunding
    _debug_fordeling        = 0       # 0: Ikke debug, 1: Debug
    _debug_fordeling_kum    = 0       # 0: Ikke debug, 1: Debug

def _debugging(ls, txt):
    print(""); print(txt)
    for i in range(len(ls)): print(ls[i] * 100)

# Legger alle x-verdiene i listen: 0, 1, 2, ... 10 (0 er ingen rette)
for i in range(n + 1): x_liste.append(i)

# Liste med fordelingen: 0, 0, 4, 11, 20, 24, 20, 11, 4, 0, 0
fordeling_liste = binom.pmf(x_liste, n, p)

# Debugger fordelingen (%): Å gjette 1 riktig betyr også å gjette 9 feil (lav sannsynlighet)
if _debug_fordeling == 1: _debugging(fordeling_liste, "Fordeling")

# Liste med kumulativ fordeling: Større eller mindre => Summere flere sannsyn
fordeling_liste_kum = np.cumsum(fordeling_liste)

# Debugger kumulativ fordeling (%): Vokser til 100 % ("mindre eller lik alle")
if _debug_fordeling_kum == 1: _debugging(fordeling_liste_kum, "Kumulativ fordeling")

# While løkken kjører så lenge sannsyn for å gjette riktig er stor (større enn konfidens)
# Bryter ut når antall riktige er så stor at det ikke er enkelt å gjette (lik konfidens)
# x_riktig starter på 0 og trekker fra økende "få" riktige fordi vi vil ha
# konfidensen til mange riktige (vanskelig å gjette riktig) på høyre side av fordelingen
while sannsyn_riktig_kum > konfidens:
    sannsyn_riktig_kum = 1 - fordeling_liste_kum[x_riktig] # Regnes ut fra "skratsj" hver gang
    x_riktig += 1
  
# Printer svaret
print(f"Må smake riktig minst {x_riktig} for at det er sannsynlig at hun smaker forskjell")



# 🚀 programmering.no | 🤓 matematikk.as
# 1T - Eksamen - 2022 Høst (Udir)
# Oppgave 3 b) Rasjonal funksjon til Lars - Kun if-blokker 

def f(x):
	return (1 - 2 * x) / (x - 2)

x = 8

while x >= -8:

    if x != 2: print(x, f(x))
    if x == 2: print(x, "Ikke definert: x = 2 deler på 0")

    x = x - 1



# 🚀 programmering.no | 🤓 matematikk.as
# - Lag et json-obj (liste med dictionaries)

# Variabler
json_obj = [
    {"fornavn": "Supermann", "alder": 25, "emoji": "💫"},
    {"fornavn": "Lynvingen", "alder": 42, "emoji": "🦇"},
    {"fornavn": "Wolverine", "alder": 31, "emoji": "🐺"},
    {"fornavn": "Spiderman", "alder": 23, "emoji": "🕸"}
]

# Iterer gjennom json_obj og print obj
for obj in json_obj:
    print(obj)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn det minste tallet i en liste med sort()

# Verdier
liste = [10, 7000, 20, 3, 400]

# Funksjon for å finne min el i en liste med sort()
def minimum_sort(liste):

	# Sorterer listen i synkende rekkefølge med reverse-argument i sort()
	liste.sort(reverse=True)

	# Min el er siste el i listen (index -1)
	min = liste[-1]

	return min

# Minimum
min = minimum_sort(liste)

# Print
print(f"Liste             : {liste}")
print(f"Minimum i listen  : {min}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om tallet på posisjon n er et fibonacci-tall
# - Fibonacci-rekken genereres av formelen F(n) = F(n-2) + F(n-1)
# - De første fibonacci-tallene er 0, 1, 1, 2, 3, 5, 8, 13, 21, ..., n

# Input
n = input("Finn fibonacci-tallet på posisjon (n): ")

# Funksjon for å finne fibonacci-tallet med for-løkke
def fibonacci_n_for_loop(n):

	# Verdier
	fib_n_1 = 0   # Det første fibonacci-tallet
	fib_n_2 = 1   # Det andre fibonacci-tallet

	# Type cast inp-str -> int
	n = int(n)

	# Ikke definert
	if n < 1:

		# Print feilmelding
		print("n (posisjon) må være et heltall")

		return "Ikke definert"

	# Det første fibonacci-tallet
	elif n == 1: return fib_n_1

	# Det andre fibonacci-tallet
	elif n == 2: return fib_n_2

	# Regn ut det n`te fibonacci-tallet med for-løkke
	else:

		# Generer det neste fibonacci-tallet for hver iterasjon
		for _ in range(2, n):
			fib_n_3 = fib_n_1 + fib_n_2
			fib_n_1 = fib_n_2
			fib_n_2 = fib_n_3

		return fib_n_2

# Fibonacci-tall
fib_n = fibonacci_n_for_loop(n)

# Print
print(f"Fibonacci-tallet på posisjon n = {n} er: {fib_n}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut summen av to tall

# Verdier
a = input("Tall 1: ")
b = input("Tall 2: ")

# Regn ut summen
c = float(a) + float(b) # Type cast inp-str -> float

# Print
print(f"Summen er: {a} + {b} = {c}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Remove (slett) element fra sett

# Variabler
sett = {"bil", "tog", "sykkel"}

# Print originalt sett
print(f"Originalt sett: {sett}")

# Remove et element fra settet
sett.remove("bil")

# Print oppdatert sett
print(f"Oppdatert sett: {sett}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 3 Coca-Cola vs. Pepsi-Cola - Oppg a), b) og c) 

from scipy.stats import binom
import numpy as np

_blokk = 1
if _blokk == 1:
    n                       = int()   # Forsøk
    p                       = 0.5     # Sannsynlighet
    x_liste                 = []      # Liste med verdiene til x
    fordeling_liste         = []      # Liste med sannsynlighetsfordelingen til x
    riktig                  = 0       # Antall riktige
    riktig_liste            = []      # Liste med antall riktige (flere enn)
    riktig_sannsyn          = float() # Sannsynligheten for å gjette X > riktige
    konfidens               = 0.05    # Konfidens
    desimal                 = 3       # Antall desimaler i avrunding
    _debug_fordeling        = 0       # 0: Ikke debug, 1: Debug
    _debug_fordeling_kum    = 0       # 0: Ikke debug, 1: Debug

# Debug fordelingen (%): Å gjette 1 riktig betyr også å gjette 9 feil (lav sannsynlighet)
def _debugging_fordeling(ls):
    if _debug_fordeling == 1:
        print(""); print("Fordeling")
        for i in range(len(ls)):
            print(ls[i] * 100) # 0, 0, 4, 11, 20, 24, 20, 11, 4, 0, 0

# Debug kumulativ fordelingen (%): Vokser til 100 % (alle gjette-kombinasjoner)
def _debugging_fordeling_kum(ls):
    if _debug_fordeling_kum == 1:
        print(""); print("Kumulativ fordeling")
        for i in range(len(riktig_liste)):
            print(riktig_liste[i] * 100)

# a) Sannsynligheten for å gjette 6 riktige
if _blokk == 1:

    # Verdier
    n = 10
    riktig = 6

    # Legger alle mulige x-verdier i listen: 0 -> 10
    for i in range(n + 1): x_liste.append(i)

    # Liste med fordelingen
    fordeling_liste = binom.pmf(x_liste, n, p)

    fordeling_liste[riktig] = round(fordeling_liste[riktig], desimal)
    print(""); print(f"a) Sannsynlighet for å gjette 6 riktig: {fordeling_liste[riktig]}")

# b) Hypotese-test
if _blokk == 1:

    # Verdier
    n = 10
    riktig = 6

    # Legger alle mulige x-verdier i listen: 0 -> 10
    for i in range(n + 1): x_liste.append(i)

    # Liste med fordelingen
    fordeling_liste = binom.pmf(x_liste, n, p)

    # Debug fordelingen (%): Å gjette 1 riktig betyr også å gjette 9 feil (lav sannsynlighet)
    _debugging_fordeling(fordeling_liste)

    # Liste med kumulative sannsynligheter (større eller mindre)
    riktig_liste = np.cumsum(fordeling_liste)

    # Sannsynligheten for å gjette riktig 8 av 10 er 94 %
    sannsyn_riktige = 1 - riktig_liste[riktig - 1]

    # Ser om kumulativ sannsynlighet for å gjette X > riktig er:
    print("")
    if sannsyn_riktige  > konfidens:
        # 1. Større enn konfidens: Stor sannsynlighet for å gjette riktig / smaker ikke forskjell
        print("b) Det er ikke sannsynlig at Marte smaker forskjell.")
    if sannsyn_riktige <= konfidens:
        # 1. Mindre enn konfidens: Liten sannsynlighet for å gjette riktig / smaker forskjell
        print("b) Det er sannsynlig at Marte smaker forskjell.")

# c) Overbevise Birger
if _blokk == 1:

    # Verdier
    n = 30
    riktig_sannsyn = 1

    # Legger alle mulige x-verdier i listen: 0 -> 10
    for i in range(n + 1): x_liste.append(i)

    # Liste med fordelingen
    fordeling_liste = binom.pmf(x_liste, n, p)

    # Debug fordelingen (%): Å gjette 1 riktig betyr også å gjette 9 feil (lav sannsynlighet)
    _debugging_fordeling(fordeling_liste)

    # Liste med kumulative sannsynligheter (større eller mindre)
    riktig_liste = np.cumsum(fordeling_liste)

    # Debug kumulativ fordelingen (%): Vokser til 100 % (alle gjette-kombinasjoner)
    _debugging_fordeling_kum(riktig_liste)

    # Sannsynligheten for å gjette riktig 8 av 10 er 94 %
    print("")
    while riktig_sannsyn > konfidens:
        riktig_sannsyn = 1 - riktig_liste[riktig]
        riktig += 1

    print(f"c) Må smake riktig minst {riktig} for at det er sannsynlig at hun smaker forskjell")




# 🚀 programmering.no | 🤓 matematikk.as
# 1T - Eksamen - 2022 Høst (Udir)
# Oppgave 3 a) Rasjonal funksjon til Lars - Finn buggen i koden til Lars 

# Funksjonen
def f(x):
	return (1 - 2 * x) / (x - 2)

x = 8

while x >= -8 :

    print(x , f(x))
    x = x - 1



# 🚀 programmering.no | 🤓 matematikk.as
# - Ser om et element eksisterer i en liste

# Nullstill
eksisterer_flagg = False

# Input
liste = input("Liste: ") # Husk [ ] og '', f.eks. ['gull', 'sølv', 'bronse']
el_inp = input("Søk etter element: ") # Uten hermetegn, f.eks. gull

# Enkel funksjon som klipper opp inp-str til liste med str-el
def str_til_str_liste(str_liste = list()):

    str_ferdig_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            str_ferdig_liste.append(el)
            el = ""
        elif str_liste[i] == ",":
            str_ferdig_liste.append(el)
            el = ""
        elif str_liste[i] == '"': pass # Fjerner doble hermetegn ("")
        elif str_liste[i] == "'": pass # Fjerner doble hermetegn ('')
        else: el += str_liste[i]
    return str_ferdig_liste

# Input-string endres til liste med str-el
liste = str_til_str_liste(liste)

# Itererer gjennom listen
for el in liste:

    # El eksisterer i listen
    if el == el_inp:

        # Sett flagget til True
        eksisterer_flagg = True

        # Print el
        print(f"Ja, {el_inp} eksisterer i listen")

# Hvis flagget fremdeles er False eksister ikke el i listen
if eksisterer_flagg == False:

    # Print el
    print("Nei, elementet eksisterer ikke i listen")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 5 a) Volum av prisme - Største volum dersom sidene i bunnen er 5 dm (geometrisk løsning) 

import random

# Konstanter
_blokk = 1 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def prisme_volum_max_fra_areal_geo(lengde = float(),
                                   dybde  = float(),
                                   A_max  = float(),
                                   rund   = 2):

    # Regner geometrisk ut volum av prisme når areal_maks, lengde og dybde er gitt

    # Skisse av kassen
    if _blokk == -1:
        #                    |------------
        #                    | .	       .
        #                    |  -	   V    -
        #                    |   .------------
        #                    _   |			 |
        #                     .  |	   A     |   hoyde
        #             dybde     _|			 |
        #                        .------------
        # 
        #                            lengde
        pass

    # Regner ut det samlede arealet til veggene 
    areal_vegger = A_max - (lengde * dybde)

    # Regner ut arealet til 1 av de 4 veggene 
    areal_vegg = areal_vegger / 4

    # Regner ut høyden fra areal_vegg = dybde * hoyde 
    hoyde = areal_vegg / dybde

    # Regner ut maksimalt volum 
    volum_max = lengde * dybde * hoyde

    # Runder av svaret
    volum_max = round(volum_max, rund)

    return volum_max

# a) Største volum dersom sidene i bunnen er 5 dm (geometrisk løsning)
if _blokk == 1:

    # Regner ut vha. den motsatte sannsynligheten
    volum_max = prisme_volum_max_fra_areal_geo(lengde = 5,
                                               dybde  = 5,
                                               A_max  = 120,
                                               rund   = 2)

    # Print svar-setninger
    svar_a_0 = f"Oppg a)"
    svar_a_1 = f"- Det største volumet kassen kan ha er {volum_max} dm^3"
    print("")
    print(svar_a_0)
    print(svar_a_1)



# 🚀 programmering.no | 🤓 matematikk.as
# Sannsynlighet (Matematikk AS)
# Mal - Sannsynlighets-relasjoner - P(X <velg relasjon> k) > sannsyn_gitt 
# - Programmet tar sikte på å kunne løse alle sannsynlighets-oppgaver som
#   - Er på prøver på VGS-nivå, del 2 (med hjelpemidler)
#   - Bruker binomisk sannsynlighets-modell/-fordeling
#     1. To utfall (enten eller)
#     2. Uavhengige delforsøk (utfallet for den ene påvirker ikke det andre)
#     3. Lik sannsynlighet for alle delforsøkene
#   - Finner sannsynlighetene gjennom simulering av mange delforsøk
#   - Veksler mellom de ulike relasjonene (==, >=, >, <= og <)
#   - Vil ofte stå på formen "Hvor mange objekter n må det være i en gruppe,
#     når det skal være en gitt sannsynlighet for at det i den samme gruppen er
#     k objekter med en binomisk fordelt egenskap"
#   - Eksempel: "Hvor mange svaner må det være i en dam for at det med minst 42 %
#     sannsynlighet skal være flere enn 4 svarte svaner i den samme dammen"
# - Skriv inn verdier og relasjoner fra oppgaven under "# Konstanter",
#   og få riktig svar som ferdige setninger i en tabell
#   - Programmet finner både n og relasjonen til n, f.eks. "minst 109"
#   - Du kan få det samme svaret som i eks. ved å bruke seed/"frø" = 1337 i
#     tilfeldighets-generatoren, np.random.default_rng(1337)
#   - Simuleringen bruker tilfeldige tall, så seed kan brukes for å få nøyaktig
#     samme sannsynlighet hver gang (de "tilfeldige" tallene i simuleringen er de samme)
#
#   ┌───────────────────────────────────────────────────┐
#   │ Dammen må ha minst 109 svaner                     │
#   ├───────────────────────────────────────────────────┤
#   │ ✓ Det skal være minst 4 svarte svaner             │
#   │ ✓ Sannsynligheten for utfallet er større enn 42 % │
#   └───────────────────────────────────────────────────┘
#
# - Tabellen nedenfor viser hvilke synonymer som kan brukes som relasjoner
#
#   ┌──────┬──────────┬──────────────────┬────────┬──────────────────┬────────┐
#   │ k    │ ==       │ >=               │ >      │ <=               │ <      │
#   │ k er │ nøyaktig │ minst            │ flere  │ maksimalt        │ mindre │
#   │      │ akkurat  │ større eller lik │ større │ mindre eller lik │ færre  │
#   │      │ lik      │                  │ mer    │                  │        │
#   └──────┴──────────┴──────────────────┴────────┴──────────────────┴────────┘

import numpy as np

# Konstanter
gruppe_navn      = "Dammen"                # Gruppens navn
obj              = "svaner"                # Alle i gruppen er samme obj
obj_p            = "svarte svaner"         # Et obj kan være obj_p eller ikke
p                = 0.03                    # Sannsynligheten for at ett obj er obj_p
k                = 4                       # Utvalget k med obj_p i gruppen
k_rel            = "minst"                 # Velg relasjon for k
n_gruppe         = 1                       # Antall obj i gruppen (starter på 1)
n_svar_rel       = str()                   # Relasjon for antall obj_p i svaret (regnes ut i koden)
n_svar_liste     = list()                  # Liste med svar-setningene
m_gruppe         = 1000                    # Mulige utfall (alle gruppe-simuleringene)
m_gruppe_liste   = list()                  # Liste med mulige utfall (fra simuleringene)
g_gruppe         = int()                   # Gunstige utfall (fra simuleringene)
sannsyn_gitt     = 0.42                    # Sannsynligheten for at k antall obj_p er i gruppen (gitt i oppg)
sannsyn_gitt_rel = "større"                # Velg relasjon for sannsyn_gitt
sannsyn_sim      = int()                   # Sannsynligheten for at k antall obj_p i gruppen (simulert)
rng              = np.random.default_rng() # Definer tilfeldighets-generatoren (random number generator)

# Innstillinger
_blokk           = 1                       # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker
_debug_while     = 1                       # 0: Av, 1: Debug while-løkke (f.eks. se sannsyn_sim-verdiene, om løkken henger)
_debug_matriks   = 0                       # 0: Av, 1: Debug riktig sannsynlighets-blokk i matriks (f.eks. trim bort ubrukte blokker, finn bugs)

# Debug while-løkke og matriks
def d(_d_s, nr):
    if _d_s == 1: print(nr)
def d_w(_d_w, sannsyn_sim):
    if _d_w == 1: print("... sannsyn_sim ::", sannsyn_sim)

# Gjør alle synonymer om til samme ord (som returneres og brukes videre internt i koden)
def synonymer(rel):
    if rel == "==" or rel == "nøyaktig"  or rel == "akkurat"          or rel == "lik": return "nøyaktig"
    if rel == ">=" or rel == "minst"     or rel == "større eller lik":                 return "minst"
    if rel == ">"  or rel == "flere"     or rel == "større"           or rel == "mer": return "flere"
    if rel == "<=" or rel == "maksimalt" or rel == "mindre eller lik":                 return "maksimalt"
    if rel == "<"  or rel == "mindre"    or rel == "færre":                            return "mindre"
    print("Feilmelding: Relasjonen er skrevet feil (er ikke i tabellen)"); return "Ikke definert"

# Formater svar-tabellen
def tabell_format(n_svar_liste):

    # Formateringen lager lange nok streker og mellomrom
    svar_lengde     = 0
    svar_liste_ny   = []

    # Iterer gjennom listen og oppdater strek og svar-lengden hvis svaret er lengre enn forrige
    for svar in n_svar_liste:
        if len(svar) > svar_lengde:
            strek = ""
            for _ in svar:
                strek += "─"
                svar_lengde = len(strek)

    # Regn ut tilsvarende lengde på mellomrommene ("padding") frem til høyre strek-marg
    def space(svar):
        space_diff = len(strek) - len(svar)
        sp = ""
        for _ in range(space_diff): sp += " "
        return sp

    # Legg de formaterte svar-setningene i en ny liste som returneres
    svar_liste_ny.append("┌─" + strek                                    + "─┐")
    svar_liste_ny.append("│ " + n_svar_liste[0] + space(n_svar_liste[0]) + " │")
    svar_liste_ny.append("├─" + strek                                    + "─┤")
    svar_liste_ny.append("│ " + n_svar_liste[1] + space(n_svar_liste[1]) + " │")
    svar_liste_ny.append("│ " + n_svar_liste[2] + space(n_svar_liste[2]) + " │")
    svar_liste_ny.append("└─" + strek                                    + "─┘")

    return svar_liste_ny

# Regner ut sannsynligheten for k antall obj_p i en gruppe med n_gruppe obj (se while-løkke)
def sannsynlighet_for(n_gruppe,
                      g_gruppe   = g_gruppe,
                      k_rel      = synonymer(k_rel),
                      s_rel      = synonymer(sannsyn_gitt_rel),
                      n_svar_rel = n_svar_rel,
                      _d_s       = _debug_matriks,
                      _d_w       = _debug_while):

    # Simulering steg-for-steg
    if _blokk == -1:
        # 1. n_gruppe starter på 1 og kan øke
        # 2. Etter første simulering sammenlignes sannsyn_sim med sannsyn_gitt
        # 3. Hvis sannsyn_sim < sannsyn_gitt øker n_klasse med 1
        # 4. Da gjøres simuleringen om igjen med den nye verdien for n_gruppe
        # 5. Simulerer at det er p sannsynlighet for at et obj er obj_p
        # 6. Dette obj blir så enten obj_p eller ikke, 1 eller 0
        # 7. Gjør dette n_gruppe ganger og får f.eks. 0, 0, 1, 0 og 1 (hvis n_gruppe er 5)
        # 8. Teller hvor mange av n_gruppe som ble obj_p, f.eks. 2
        # 9. Legger 2 i listen, [2]
        # 10. Gjør dette med m_gruppe grupper, [2, 0, 0, ..., 1]
        # 11. m_gruppe_liste får dermed alle utfallene (fra simuleringen)
        pass
    m_gruppe_liste = rng.binomial(n = n_gruppe, p = p, size = m_gruppe)

    # Matriks steg-for-steg
    if _blokk == 1:
        if _blokk == -1:
            # 1. Når du leser setningen "Det er_________x % sannsyn..." (x = sannsyn_gitt) nedenfor,
            #   så kan du tenke at du "fyller inn" relasjonene som står i kolonnene under
            #   (relasjonene som du først skrev inn som konstanter i starten av koden)
            # 2. Da kan du lese av hele den aktuelle raden (riktig kombinasjon for dine relasjoner) og se
            #    2.1. Hvilke relasjon som returneres til ditt svar (hvis oppg spør om hvor stor n_gruppe med obj
            #         må være for å få ønsket sannsynlighet for å tilfeldig trekke k obj_p fra gruppen
            #    2.2. Hvilken sannsynlighets-blokk som brukes (hvis du setter _debug_sannsyn = 1), f.eks. "3-2"
            pass
        #          Det er_________x % sannsyn for__________(enn) k obj_p i gruppen
        if _blokk == 1:
            if s_rel == "nøyaktig"  and k_rel == "nøyaktig":   g_gruppe = sum(m_gruppe_liste == k); d(_d_s, "1-1"); n_svar_rel = "nøyaktig"
            if s_rel == "nøyaktig"  and k_rel == "minst":      g_gruppe = sum(m_gruppe_liste >= k); d(_d_s, "1-2"); n_svar_rel = "nøyaktig"
            if s_rel == "nøyaktig"  and k_rel == "flere":      g_gruppe = sum(m_gruppe_liste >  k); d(_d_s, "1-3"); n_svar_rel = "nøyaktig"
            if s_rel == "nøyaktig"  and k_rel == "maksimalt":  g_gruppe = sum(m_gruppe_liste <= k); d(_d_s, "1-4"); n_svar_rel = "nøyaktig"
            if s_rel == "nøyaktig"  and k_rel == "mindre":     g_gruppe = sum(m_gruppe_liste <  k); d(_d_s, "1-5"); n_svar_rel = "nøyaktig"
        if _blokk == 1:
            if s_rel == "minst"     and k_rel == "nøyaktig":   g_gruppe = sum(m_gruppe_liste == k); d(_d_s, "2-1"); n_svar_rel = "minst"
            if s_rel == "minst"     and k_rel == "minst":      g_gruppe = sum(m_gruppe_liste >= k); d(_d_s, "2-2"); n_svar_rel = "minst"
            if s_rel == "minst"     and k_rel == "flere":      g_gruppe = sum(m_gruppe_liste >  k); d(_d_s, "2-3"); n_svar_rel = "flere"
            if s_rel == "minst"     and k_rel == "maksimalt":  g_gruppe = sum(m_gruppe_liste <= k); d(_d_s, "2-4"); n_svar_rel = "maksimalt"
            if s_rel == "minst"     and k_rel == "mindre":     g_gruppe = sum(m_gruppe_liste <  k); d(_d_s, "2-5"); n_svar_rel = "mindre"
        if _blokk == 1:
            if s_rel == "flere"     and k_rel == "nøyaktig":   g_gruppe = sum(m_gruppe_liste == k); d(_d_s, "3-1"); n_svar_rel = "flere"
            if s_rel == "flere"     and k_rel == "minst":      g_gruppe = sum(m_gruppe_liste >= k); d(_d_s, "3-2"); n_svar_rel = "minst"
            if s_rel == "flere"     and k_rel == "flere":      g_gruppe = sum(m_gruppe_liste >  k); d(_d_s, "3-3"); n_svar_rel = "flere"
            if s_rel == "flere"     and k_rel == "maksimalt":  g_gruppe = sum(m_gruppe_liste <= k); d(_d_s, "3-4"); n_svar_rel = "maksimalt"
            if s_rel == "flere"     and k_rel == "mindre":     g_gruppe = sum(m_gruppe_liste <  k); d(_d_s, "3-5"); n_svar_rel = "mindre"
        if _blokk == 1:
            if s_rel == "maksimalt" and k_rel == "nøyaktig":   g_gruppe = sum(m_gruppe_liste == k); d(_d_s, "4-1"); n_svar_rel = "maksimalt"
            if s_rel == "maksimalt" and k_rel == "minst":      g_gruppe = sum(m_gruppe_liste >= k); d(_d_s, "4-2"); n_svar_rel = "maksimalt"
            if s_rel == "maksimalt" and k_rel == "flere":      g_gruppe = sum(m_gruppe_liste >  k); d(_d_s, "4-3"); n_svar_rel = "maksimalt"
            if s_rel == "maksimalt" and k_rel == "maksimalt":  g_gruppe = sum(m_gruppe_liste <= k); d(_d_s, "4-4"); n_svar_rel = "minst"
            if s_rel == "maksimalt" and k_rel == "mindre":     g_gruppe = sum(m_gruppe_liste <  k); d(_d_s, "4-5"); n_svar_rel = "minst"
        if _blokk == 1:
            if s_rel == "mindre"    and k_rel == "nøyaktig":   g_gruppe = sum(m_gruppe_liste == k); d(_d_s, "5-1"); n_svar_rel = "mindre"
            if s_rel == "mindre"    and k_rel == "minst":      g_gruppe = sum(m_gruppe_liste >= k); d(_d_s, "5-2"); n_svar_rel = "maksimalt"
            if s_rel == "mindre"    and k_rel == "flere":      g_gruppe = sum(m_gruppe_liste >  k); d(_d_s, "5-3"); n_svar_rel = "minst"
            if s_rel == "mindre"    and k_rel == "maksimalt":  g_gruppe = sum(m_gruppe_liste <= k); d(_d_s, "5-4"); n_svar_rel = "minst"
            if s_rel == "mindre"    and k_rel == "mindre":     g_gruppe = sum(m_gruppe_liste <  k); d(_d_s, "5-5"); n_svar_rel = "minst"

    # Definisjonen av sannsynlighet gir P(OP) = g / m, der hendelsen OP: "obj i gruppen er obj_p"
    sannsyn_sim = g_gruppe / m_gruppe

    # 0: Av, 1: Debug while-løkke (f.eks. se sannsyn_sim-verdiene, om løkken henger)
    d_w(_d_w, f"{sannsyn_sim}")

    # Returnerer en liste med sannsynlighet og relasjon
    return [sannsyn_sim, n_svar_rel]

# While-løkke steg-for-steg
if _blokk == -1:
    # 1. Betingelsen i løkken er to-delt
    #    1.1 For ">=", ">", "==": sannsyn_sim går fra 0 opp til sannsyn_gitt
    #    1.2 For "<=", "<"      : sannsyn_sim går fra 1 ned til sannsyn_sim
    # 2. Løkken kaller f_sannsynlighet_for() med to arg
    #    2.1. sannsyn_gitt_rel (valgfri/har default): ("==", ">=", ">", "<=", "<")
    #    2.2. n_klasse (obligatorisk)               : nåværende gutter i klassen
    # 3. sannsynlighet_for() returnerer to variabler (i en liste)
    #    3.1 sannsynlighet_for()[0]: sannsyn_sim
    #    3.2 sannsynlighet_for()[1]: svar_rel (relasjonen i svaret)
    # 4. Hvis f.eks. sannsyn_sim < sannsyn_gitt (betingelsen), så øker n_gruppe med 1
    # 5. Isåfall kaller løkken sannsynlighet_for() om igjen
    # 6. Løkken vil da repeterer dette til sannsyn_sim >= sannsyn_gitt (betingelsen brytes)
    # 7. Merk at == ikke kan brukes i sannsynlighet_for(n_gruppe)[0] == sannsyn_gitt
    #   fordi sannsyn_sim aldri starter på sannsyn_gitt (betingelsen brytes isåfall med en gang)
    #   == erstattes derfor med <=, som blir beste tilnærming (bryter betingelsen med en
    #   sannsyn_sim er rett over, eller teoretisk sett lik)
    pass
while ((synonymer(sannsyn_gitt_rel)    == "nøyaktig"    and (k_rel == "nøyaktig"  or k_rel == "minst"   or k_rel == "flere") and
        sannsynlighet_for(n_gruppe)[0] <= sannsyn_gitt) or # == er byttet ut med <= (se kommentar over)
       (synonymer(sannsyn_gitt_rel)    == "nøyaktig"    and (k_rel == "maksimalt" or k_rel == "mindre") and
        sannsynlighet_for(n_gruppe)[0] >= sannsyn_gitt) or

       (synonymer(sannsyn_gitt_rel)    == "minst"       and (k_rel == "nøyaktig"  or k_rel == "minst"   or k_rel == "flere") and
        sannsynlighet_for(n_gruppe)[0] <= sannsyn_gitt) or
       (synonymer(sannsyn_gitt_rel)    == "minst"       and (k_rel == "maksimalt" or k_rel == "mindre") and
        sannsynlighet_for(n_gruppe)[0] >= sannsyn_gitt) or

       (synonymer(sannsyn_gitt_rel)    == "flere"       and (k_rel == "nøyaktig"  or k_rel == "minst"   or k_rel == "flere") and
        sannsynlighet_for(n_gruppe)[0] <  sannsyn_gitt) or
       (synonymer(sannsyn_gitt_rel)    == "flere"       and (k_rel == "maksimalt" or k_rel == "mindre") and
        sannsynlighet_for(n_gruppe)[0] >  sannsyn_gitt) or

       (synonymer(sannsyn_gitt_rel)    == "maksimalt"   and (k_rel == "nøyaktig"  or k_rel == "minst"   or k_rel == "flere") and
        sannsynlighet_for(n_gruppe)[0] <= sannsyn_gitt) or
       (synonymer(sannsyn_gitt_rel)    == "maksimalt"   and (k_rel == "maksimalt" or k_rel == "mindre") and
        sannsynlighet_for(n_gruppe)[0] >= sannsyn_gitt) or

       (synonymer(sannsyn_gitt_rel)    == "mindre"      and (k_rel == "nøyaktig"  or k_rel == "minst"   or k_rel == "flere") and
        sannsynlighet_for(n_gruppe)[0] <  sannsyn_gitt) or
       (synonymer(sannsyn_gitt_rel)    == "mindre"      and (k_rel == "maksimalt" or k_rel == "mindre") and
        sannsynlighet_for(n_gruppe)[0] >  sannsyn_gitt)):

    # n_gruppe øker med 1 hver gang løkken (og simuleringen) kjøres
    n_gruppe += 1

# Kaller sannsynlighet_for() en ekstra gang med forrige klasse_n for å få relasjonen til svaret
n_svar_rel = sannsynlighet_for(n_gruppe - 1, sannsyn_gitt_rel)[1]

# Returner "enn" for relasjonene i if-blokkene og "" for andre
def enn_rel(rel):
    if rel == "flere" or rel == "større" or rel == "mer" or rel == "mindre" or rel == "færre": return f"{rel} enn "
    else: return f"{rel} "

# Gang med 100 for prosent og rund av til f.eks. 1 eller 0 (None) desimaler
sannsyn_gitt = round(sannsyn_gitt * 100, None)

# Legg svar-setningene i en liste og formater
n_svar_liste = [
        f"{gruppe_navn} må ha {enn_rel(n_svar_rel)}{n_gruppe} {obj} ",
        f"✓ Det skal være {enn_rel(k_rel)}{k} {obj_p}",
        f"✓ Sannsynligheten for utfallet er {enn_rel(sannsyn_gitt_rel)}{sannsyn_gitt} %"]
n_svar_liste = tabell_format(n_svar_liste)

# Print svar-setninger
for svar in n_svar_liste:
    print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 4 a) Sannsynlighet med 5 terninger - Bruk programmering til å bestemme P(X > 20) 

import numpy as np

# definerer tilfeldighetsgeneratoren
rng = np.random.default_rng()

# tal simuleringar
N = 10000000

# triller 5 terningar N gongar
terningar = rng.integers(1, 7, size=(5, N))

# finn summen av kvart forsøk
fem_kast = terningar.sum(axis=0)

# tel opp kor mange gongar summen vart større enn 20
gunstige = sum(fem_kast > 20)

# finn sannsynet og skriv ut
p = gunstige / N

print(f"P(X > 20) = {p:.4f}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 5 a) Karakter-statestikk - Sannsynligheten for et karakter-snitt over karakter 4.0 

import numpy as np 
def trekk(n):
    E=[]     #liste for elevenes karakterer
    for i in range(n):
        s = np.random.randint(3) #trekker en av tre skoler
        if s == 0:
            E.append(np.random.normal(3.8,1.2))
        elif s == 1:
            E.append(np.random.normal(3.4,1.4))
        else:
            E.append(np.random.normal(4.1,1.1))
    return np.mean(E)

N = 10000
merennfire = 0
i=0

for i in range(N):
    if trekk(20)>4:
        merennfire = merennfire + 1
    i = i +1

print(f"Sannsynligheten for at snittkarakteren er mer enn fire, er {100*merennfire/N:.1f} prosent. ")



# 🚀 programmering.no | 🤓 matematikk.as
# - Transponer matrise med numpy

import numpy

# Verdier
matrise = [[1, 2, 3], [4, 5, 6]]

# Transponer matrisen
matrise_transpose = numpy.transpose(matrise)

# Print
print("")
print("Matrise:")
print(matrise)

print("")
print("Transponert matrise:")
print(matrise_transpose)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn positive tall i en liste

# Verdier
liste = [-3, -2.5, -1, 0, 1, 2, 3, 4, 5, 6]

# Funksjon for å finne positive tall i en liste
def positive_tall_liste(liste):

	# Nullstill
	positive_tall_liste = list()

	# Itererer gjennom listen
	for tall in liste:

		# Legg tallet i listen hvis det er positivt
		if tall > 0:
			positive_tall_liste.append(tall)

	return positive_tall_liste

# Positive tall
positive_tall_liste = positive_tall_liste(liste)

# Print
print(f"Original liste          : {liste}")
print(f"Liste med positive tall : {positive_tall_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 5 a) Karakter-statestikk - Normalfordelt karakter-snitt til 20 elever fra 3 skoler 

import numpy as np

_blokk = 1
if _blokk == 1:
    elev_ant        = 20        # Antall elever vi trekker
    elev_kar        = float()   # Gjenommsnitts-karakter til en elev
    elev_liste      = list()    # Liste med gjenommsnitts-karakterer til elevene
    elev_snitt      = float()   # Snittet av alle elevene
    skole_ant       = 3         # Antall skoler
    skole_a_mu      = 3.8       # Skole A: Forventningsverdi (mu)
    skole_a_st      = 1.2       # Skole A: Standardavvik (st)
    skole_b_mu      = 3.4       # Skole B: Forventningsverdi (mu)
    skole_b_st      = 1.4       # Skole B: Standardavvik (st)
    skole_c_mu      = 4.1       # Skole C: Forventningsverdi (mu)
    skole_c_st      = 1.1       # Skole C: Standardavvik (st)
    desimal         = 2         # Avrunding

# For hver av 20 elevene
for i in range(elev_ant):

    # Trekker tilfeldig skole
    skole_r = np.random.randint(skole_ant)

    # Vi får gjennomsnitskarakter fra numpy sin normalfordeling
    if skole_r == 0: elev_kar = np.random.normal(skole_a_mu, skole_a_st)
    if skole_r == 1: elev_kar = np.random.normal(skole_b_mu, skole_b_st)
    if skole_r == 2: elev_kar = np.random.normal(skole_c_mu, skole_c_st)

    # Liste med gjenommsnitts-karakterer til elevene 
    elev_liste.append(elev_kar)

# Snittet av alle elevene
elev_snitt = round(np.mean(elev_liste), desimal)

# Print svar
print(f"Gjennomsnitts-karakteren er: {elev_snitt}")




# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 4 a) Sannsynlighet med 5 terninger - Bruk programmering til å bestemme P(X > 20) 

import random

# Konstanter
_blokk = 1 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# Definerer funksjoner
def terninger_minst_2_like_av_n(n_terninger = int(),
                                svar_typ    = "desimaltall",
                                rund        = 2):

    # Definert for maksimalt 5 terninger

    # Ingen like: Gunstige
    g_ingen_like    = 1                 # (*) Starter på 1 pga. ganges med seg selv i uttrykket 6/6 * 5/6 * ...
    g_sider         = 6                 # Alle terningene har 6 sider
    terning_nr      = 1                 # Den 1. terningen av n_terninger

    # Løkken steg-for-steg: lager uttrykket 6/6 * 5/6 * 4/3 ... med alle terningene (n_terninger)
    while terning_nr <= n_terninger:    # Steg 1: Betingelsen sjekker om vi har flere terninger
        g_ingen_like    *= g_sider      # Steg 2: (***) Ganges med seg selv hver ganng g_sider minker med 1, 6/6 * 5/6 * ...
        g_sider         -= 1            # Steg 3: g_sider minker med en for hver terning
        terning_nr      += 1            # Steg 4: terning_nr øker med 1 så lenge <= n_terninger (betingelsen)

    # Ingen like: Mulige
    m_ingen_like = 6**n_terninger

    # Definisjonen av sannsynlighet gir P(2L) = g / m, der hendelsen 2L: "Minst to like"
    sannsyn_ingen_like = g_ingen_like / m_ingen_like

    # Minst 2 like (den motsatte sannsynligheten)
    sannsyn_minst_to_like = 1 - sannsyn_ingen_like

    # Velg mellom svar som desimaltall eller prosent
        if svar_typ == "prosent": sannsyn_minst_to_like *= 100
        if svar_typ == "rund"   : pass

    # Runder av svaret
    sannsyn_minst_to_like = round(sannsyn_minst_to_like, rund)

    return sannsyn_minst_to_like

def terninger_oyne_sum_fra_n_storre_enn_k(n_terninger = int(),
                                          oyne_sum    = int(),
                                          svar_typ    = "prosent",
                                          rund        = 2):

    oyne_sum_liste  = list()      # Liste med resultatet for hver simulering
    m_terning_sim   = 10000       # Antall simuleringer (antall mulige)

    # For-løkken steg-for-steg: Legger et tilfeldig antall øyne i listen for hver iterasjon
    for sim in range(m_terning_sim):                    # Iterer gjennom alle elementene i m_terning_sim
        oyne_sum_sim = 0                                # Nullstiller oyne_sum_sim før hver iterasjon
        for i in range(n_terninger):                    # Iterer gjennom antall terninger
            terning_kast = random.randint(1, 6)         # Simulert kast for hver terning gir et tall mellom 1 og 6
            oyne_sum_sim += terning_kast                # For alle terningene legges antall øyne til oyne_sum_sim
            # if oyne_sum_sim == 30: print(Yatzee!)     # Kan debugge løkken når vi f.eks. får yatzee med 6'ere

        # Legg summen av øynene i fra hver simulering i listen
        oyne_sum_liste.append(oyne_sum_sim)

    # Tell antall gunstige summer i listen
    g_terning = 0
    for terning in oyne_sum_liste:
        if terning > oyne_sum:
            g_terning += 1

    # Definisjonen av sannsynlighet gir P(X >= 20) = g / m, der hendelsen X: "Summen av øyne er større enn 20"
    sannsyn_terning = g_terning / m_terning_sim # print(sannsyn_terning) # Debug sannsynligheten når løkken kjører

    # Velg mellom svar som desimaltall eller prosent
        if svar_typ == "prosent": sannsyn_terning *= 100
        if svar_typ == "rund"   : pass

    # Runder av svaret
    sannsyn_terning = round(sannsyn_terning, rund)

    return sannsyn_terning

# a) Sannsynligheten for minst 2 av 5 terningene er like
if _blokk == 1:

    sannsyn_minst_to_like = terninger_minst_2_like_av_n(n_terninger = 5,
                                                        svar_typ    = "prosent",
                                                        rund        = 2)

    # Print svar-setninger
    svar_a_0 = f"Oppg a)"
    svar_a_1 = f"- Sannsynligheten for minst 2 like av 5 terninger er {sannsyn_minst_to_like} %"
    print("")
    print(svar_a_0)
    print(svar_a_1)

# b) Bruk programmering til å bestemme P(X > 20)
if _blokk == 1:

    sannsyn_terning = terninger_oyne_sum_fra_n_storre_enn_k(n_terninger = 5,
                                                            svar_typ    = "prosent",
                                                            rund        = 2)

    # Print svar-setninger
    svar_b_0 = f"Oppg b)"
    svar_b_1 = f"- P(x > 20) = {sannsyn_terning} %"
    print("")
    print(svar_b_0)
    print(svar_b_1)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn det minste tallet i en liste med for-løkke

# Input
liste = [2, 3, 400, 50, 10]

# Funksjon for å finne min el i en liste med for-løkke
def minimum_for_loop(liste):

	# Sett første el som forløpig min
	min = liste[0]

	# Itererer gjennom listen
	for i in range(1, len(liste)):

		# Oppdater min hvis el er mindre enn nåværende min
		if liste[i] < min:
			min = liste[i]

	return min

# Minimum
min = minimum_for_loop(liste)

# Print
print(f"Liste             : {liste}")
print(f"Minimum i listen  : {min}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 b) Sofa-produksjon til møbelfabrikk - Størst overskudd 
# - Løser oppgaven med CAS i Python
# - Kopier kommandoen i terminalen og importer matematikk 
#   $ pip install matematikk

from matematikk import overskudd_max, reggis, Symbol

# a) Finn O(x)
K = reggis(variabel = Symbol("x"),
           grad     = 2,
           x_liste  = [10, 25, 40, 70, 100, 140, 180],
           y_liste  = [270, 550, 870, 1500, 2200, 3300, 4500],
           rund     = 2)
print(K)

# b) Størst overskudd
x_max = overskudd_max(variabel = Symbol("x"),
                      uttrykk  = -0.041*Symbol("x")**2 + 11*Symbol("x") - 103,
                      rund     = None,
                      debug    = -1)
print(x_max)





# 🚀 programmering.no | 🤓 matematikk.as
# - Kopier liste med deepcopy()

from copy import deepcopy

# Verdier
liste_1 = [{"Navn": "Clark"}]
liste_2 = [{"Navn": "Clark"}]

# Kopier liste med copy()
liste_copy_1 = liste_1.copy()

# Endre feltet til en ny verdi
liste_1[0]["Navn"] = "Superman"

# Begge listene ble endret (er i praksis samme liste)
print(f"Liste 1 - copy()")
print(f"Original liste : {liste_1}")
print(f"Kopiert liste  : {liste_copy_1}")

# Kopier liste med deepcopy()
liste_copy_2 = deepcopy(liste_2)

# Endre feltet til en ny verdi
liste_2[0]["Navn"] = "Superman"

# Kun den ene listen ble endret (er to uavhengige lister)
print(f"Liste 2 - deepcopy()")
print(f"Original liste : {liste_2}")
print(f"Kopiert liste  : {liste_copy_2}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 5 Karakter-statestikk - Oppg a) og b) 
import numpy as np

_blokk = 1
if _blokk == 1:
    elev_ant    = 20        # Antall elever som trekkes
    elev_kar    = float()   # Gjennomsnitts-karakteren til tilfeldig elev
    elev_liste  = list()    # Liste for elevenes karakterer
    elev_snitt  = float()   # Karaktersnittet til elevene
    skole_ant   = 3         # Antall skoler
    skole_r     = int()     # Tilfeldig trukket skole
    skole_a_mu  = 3.8       # Skole A: Forventingsverdi (mu)
    skole_b_mu  = 3.4       # Skole B: Forventingsverdi (mu)
    skole_c_mu  = 4.1       # Skole C: Forventingsverdi (mu)
    skole_a_st  = 1.2       # Skole A: Standardavvik (sigma)
    skole_b_st  = 1.4       # Skole B: Standardavvik (sigma)
    skole_c_st  = 1.1       # Skole C: Standardavvik (sigma)
    kar_grense  = 4.0       # Karakter-grense
    kar_gunstig = 0         # Antall topp-karakter
    sim_ant     = 10000     # Antall simulering
    sannyn_kar  = float()   # Sannsynlighet for topp-karakter
    desimal     = 2         # Avrunding

# Funksjon for snitt
def elev_karaktersnitt(n):

    # Resetting
    elev_liste = []

    # For hver av de 20 elevene
    for i in range(n):

        # Trekker eleven tilfeldig fra 1 av 3 skoler
        skole_r = np.random.randint(skole_ant)

        # Regner ut gjennomsnitts-karakteren med numpy
        if skole_r == 0: elev_kar = np.random.normal(skole_a_mu, skole_a_st)
        if skole_r == 1: elev_kar = np.random.normal(skole_b_mu, skole_b_st)
        if skole_r == 2: elev_kar = np.random.normal(skole_c_mu, skole_c_st)

        # Legger karakteren i karakter-listen
        elev_liste.append(elev_kar)

    # Regner ut gjennomsnittet til de 20 elevene
    elev_snitt = round(np.mean(elev_liste), desimal)

    return elev_snitt

# Oppg a)
if _blokk == 1:

    # Snittet av alle elevene
    elev_snitt = elev_karaktersnitt(elev_ant)

    # Print svar
    print("")
    print("Oppg a)")
    print(f"Gjennomsnitts-karakteren er: {elev_snitt}")
 
# Oppg b)
if _blokk == 1:

    # Simulering
    for i in range(sim_ant):

        # Snittet fra n elever
        elev_snitt = elev_karaktersnitt(elev_ant)

        # Øk med en for hver topp-karakter
        if elev_snitt > kar_grense: kar_gunstig += 1

    # Sannsynlighet for topp-karakter: P = g / m
    sannyn_kar = (kar_gunstig / sim_ant) * 100

    # Avrunding
    sannyn_kar = round(sannyn_kar, desimal)

    # Print svar
    print("")
    print("Oppg b)")
    print(f"Sannsynlighet for {kar_grense} er: {sannyn_kar} %")



# 🚀 programmering.no | 🤓 matematikk.as
# - Eksempel på complex (komplekse tall)

# Variabler
komplekst_tall_1 = 2 + 4j
komplekst_tall_2 = 3 + 5j

# Finn summen av to komplekste tall
komplekst_tall_sum = komplekst_tall_1 + komplekst_tall_2

# Print
print(komplekst_tall_sum)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn det største elementet i en liste med sort()

# Input
liste = input("Finn maksimum i listen: ") # Husk [ ], f.eks. [1, 2, 3, 4]

# Enkel funksjon som klipper opp inp-str til liste med int-el
def str_til_int_liste(str_liste = list()):

    int_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            int_liste.append(int(el))
            el = ""
        elif str_liste[i] == ",":
            int_liste.append(int(el))
            el = ""
        else: el += str_liste[i]
    return int_liste

# Funksjon for å finne max el i en liste med sort()
def maksimum_sort(liste):

	# Sorterer listen i stigende rekkefølge
	liste.sort()

	# Max el er siste el i listen (index -1)
	max = liste[-1]

	return max

# Input-string endres til liste med int-el
liste = str_til_int_liste(liste)

# Maksimum
max = maksimum_sort(liste)

# Print
print(f"Liste             : {liste}")
print(f"Maksimum i listen : {max}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om tallet på posisjon n er et fibonacci-tall
# - Fibonacci-rekken genereres av formelen F(n) = F(n-2) + F(n-1)
# - De første fibonacci-tallene er 0, 1, 1, 2, 3, 5, 8, 13, 21, ..., n
# - Newtons metode definerer det første fibonacci-tallet som 1 (og ikke 0)
#   Vi kan forskyve rekken med m = n - 1 slik at dette likevel blir 0

from math import sqrt

# Verdier
n = 13

# Funksjon for å finne fibonacci-tallet med Newtons metode
def fibonacci_n_newton(n):

	# Verdier
	fib_n_1 = 0 # Det første fibonacci-tallet

	# Type cast inp-str -> int
	n = int(n)

	# Ikke definert
	if n < 1:

		# Print feilmelding
		print("n (posisjon) må være et heltall")

		return "Ikke definert"

	# Det første fibonacci-tallet
	elif n == 1: return fib_n_1

	# Regn ut det n`te fibonacci-tallet med Newtons metode
	else:

		# Posisjonen til det forrige fibonacci-tallet
		m = n - 1

		# Formel for Newtons metode
		fib_n = ((1 + sqrt(5))**m - (1 - sqrt(5))**m) / (2**m*sqrt(5))

		return int(fib_n)

# Fibonacci-tall
fib_n = fibonacci_n_newton(n)

# Print
print(f"Fibonacci-tallet på posisjon n = {n} er: {fib_n}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om tallet på posisjon n er et fibonacci-tall
# - Fibonacci-rekken genereres av formelen F(n) = F(n-2) + F(n-1)
# - De første fibonacci-tallene er 0, 1, 1, 2, 3, 5, 8, 13, 21, ..., n

# Verdier
fib_1 = 0   # Det første fibonacci-tallet
fib_2 = 1   # Det andre fibonacci-tallet

# Input
n = 8

# Rekursiv fibonacci-funksjon
def fibonacci_n_rekursiv(n):

	# Type cast inp-str -> int
	n = int(n)

	# Ikke definert
	if n < 1:

		# Print feilmelding
		print("n (posisjon) må være et heltall")

		return "Ikke definert"

	# Det første fibonacci-tallet
	elif n == 1: return fib_1

	# Det andre fibonacci-tallet
	elif n == 2: return fib_2

	# Regn ut det n`te fibonacci-tallet rekursivt
	else: return fibonacci_n_rekursiv(n - 1) + fibonacci_n_rekursiv(n - 2)

# Fibonacci-tall
fib_n = fibonacci_n_rekursiv(n)

# Print
print(f"Fibonacci-tallet på posisjon n = {n} er: {fib_n}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 5 a) Sannsynlighet med 5 terninger - Sannsynligheten for minst 2 av 5 terningene er like 

# gunstige / moglege utfall
ingen_like = (6*5*4*3*2) / (6**5)

minst_to_like = 1 - ingen_like

print(f"P(minst to like) = {minst_to_like:.4f}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn antall elementer i en liste med len()

# Verdier
liste = ["Merkus", "Venus", "Jorden", "Mars"]

# Print
print(f"Liste: {liste}")

# Antall el i listen ("lengden" til listen)
el_ant = len(liste)

# Print
print(f"Antall el i listen: {el_ant}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 6 a) Tredjegradsfunksjon - Påstand 1: Grafen f har minst ett ekstremalpunkt 

from sympy import symbols, diff, solve

# Dersom grafen til f har et vendepunkt for x = 3 er f'(1) = f'(5)

# Dersom grafen til f har vendepunkt for x = 3 vil det si at f''(3) = 0
# Siden f''(x) er den deriverte til f'(x) vet vi da at f'(x) har
# ekstremalpunkt for x = 3. Siden f'(x) er et tredjegradspolynom er f'(x)
# et andregradspolynom.
# Andregradspolynomer er symmetriske om ekstremalpunktet sitt.
# Siden x = 1 og x = 5 ligger like langt fra x = 3 vil f'(1) = f'(5)
#
# Påstanden er sann



# 🚀 programmering.no | 🤓 matematikk.as
# - Ugyldige variabelnavn

# Variabler
1variabel    = "Funker koden?"
en variabel  = "Hva med dette?"
spesialtegn$ = "Hvorfor funker ikke koden?"

# Print
print(spesialtegn$)



# 🚀 programmering.no | 🤓 matematikk.as
# - Endre value med key

# Variabler
obj = {
    "navn":  "Supermann",
    "alder": 23
}

# Print obj
print(obj)

# Legger til et nytt felt fordi key ikke eksisterer
obj["emoji"] = "💫"

# Print oppdatert obj
print(obj)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn oddetallene i en liste

# Verdier
liste = [-3, -2.5, -1, 0, 1, 2, 3, 4, 5, 6]

# Funksjon for å finner oddetallene i en liste
def oddetall_liste_mod(liste):

	# Nullstill
	oddetall_liste = list()

	# Itererer gjennom listen
	for tall in liste:

		# Hvis resten av divisjon med 2 ikke er 0, så er tallet et oddetall
		if tall % 2 != 0:
			oddetall_liste.append(tall)

	return oddetall_liste

# Oddetatall-listen
oddetall_liste = oddetall_liste_mod(liste)

# Print
print(f"Original liste : {liste}")
print(f"Oddetall-liste  : {oddetall_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om intervallet har primtall
# - Algoritmen kan optimaliseres ved å bytte ut int(n / 2) med int(sqrt(n))

# Variabler
primtall_liste = list()

# Input
min = input("Min verdi: ")
max = input("Max verdi: ")

# Funksjon for å se intervallet har primtall
def er_primtall_intervall(min, max):

    # Itererer gjennom intervallet
	for n in range(int(min), int(max) + 1):

		# Alle primtall er større enn 1
		if n > 1:

			# Iterer fra 2 til n / 2 + 1 (medium tempo)
			for i in range(2, int(n / 2) + 1):

				# n er ikke et primtall hvis det er delelig med i
				if n % i == 0: break

			# Hvis for-løkken fullføres uten break
			else: primtall_liste.append(n)

	return primtall_liste

# Primtall eller ikke
primtall_liste = er_primtall_intervall(min, max)

# Print
if len(primtall_liste) == 0:
	print(f"Det er ingen primtall mellom {min} og {max}")
else:
	print(f"Primtallene mellom {min} og {max} er: {primtall_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Fjern alle positive tall fra en liste med remove()

# Verdier
liste = [-3, -2.5, -1, 0, 1, 2, 3, 4, 5, 6]

# Funksjon som fjerner positive tall fra en liste
def fjern_positive_tall(liste):

	# Nullstill
	negative_tall_liste = list()

	# Itererer gjennom listen
	for tall in liste:

		# Legg tallet i listen hvis det er negativt
		if tall < 0:
			negative_tall_liste.append(tall)

	return negative_tall_liste

# Liste med positive tall
negative_tall_liste = fjern_positive_tall(liste)

# Print
print(f"Original liste          : {liste}")
print(f"Liste med negative tall : {negative_tall_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Endre value med key

# Variabler
obj = {
    "navn":  "Supermann",
    "alder": 23
}

# Endre value med key
obj["navn"] = "Batman"

# Print oppdatert obj
print(obj)

# Print oppdatert navn
print(obj["navn"])



# 🚀 programmering.no | 🤓 matematikk.as
# - Reverserer rekkefølgen til elementene i en liste
# - Hastighet reverse(): Ca. 33 ms per loop for liste m/ 100 000 000 el

# Variabler
liste = [1, 2, 3, 4]

# Print
print(f"Original liste  : {liste}")

# Reverse
liste.reverse()

# Print
print(f"Reversert liste : {liste}")
